# STEM Projects | Instructor: G. Lindley | EET-085-49563-2022SP | Lake Land College (LLC) - Mattoon, IL
# Interface: Raspberry Pi Pico (2020) w/ MicroPython & Thonny IDE
# Project:   NAME! THAT! GATE!
# By:        Krietemeyer, Broc
# Date:      2022 / Apr / 20
#
# Description:
#    This is a game based off of another (Name That Gate! 2) found in the ETCAI Software package (called
# (Digital Challenge V5.0) which was commonly used during my electronics studies (Applied Engineering -
# Industrial Computer Systems) in the Technology Division at LLC. Essentially, it's a test of Digital
# Logic Gate operation when given its' inputs and their logic level if the Gate itself is unknown. There
# are 140pts possible, each stage is worth 14pts, there are x10 stages, each input toggle deducts x1 point
# from a stage, correct Gate selection rewards all remaining stage points, and an incorrect Gate selection
# deducts all remaining stage points.
#
#    Gameplay:
#        1) From startup, x1 Logic Gate (of 10 possible) is randomly loaded.
#            - Logic Gates are:  2-inputs - AND, NAND, NOR, OR, XOR, XNOR
#                                3-inputs - AND, NAND, NOR, OR
#        2) Inputs always start LOW.
#            - If only a 2 input Gate is loaded then Inp(C) is deactivated & invalid.
#        3) Initial Gate output reflects what is TRUE for the logic of that loaded Gate.
#        4) From this point, the player can either toggle any of the inputs or select a Logic Gate.
#            - If input toggled: Even if Inp(C) invalid, x1 stage point deducted if any of 14pts remaining.
#            - If Gate right:    All remaining stage points remain and next Gate (if any) loaded.
#            - If Gate wrong:    All remaining stage points deducted and next Gate (if any) loaded.
#        5) Loaded Gate output will dynamically reflect what is TRUE given the supplied inputs.
#        6) Once a Gate selection is made the stage ends and a new Logic Gate is randomly loaded.
#            - No same Gate is encountered twice during a full (stage 1 - 10) playthrough.
#        7) After stage end, inputs are set LOW again and initial Gate output reflects what is TRUE.
#        8) Again, player can either toggle any of the inputs or select a Logic Gate.
#        9) When no new Gates (of 10) remain, then game ends.


######## SECTION 1 of 7: Importing Libraries & Classes ########

from machine import I2C, Pin # imports I2C & Pin class from machine module
                                    # MicroPython includes machine module by default
from pico_i2c_lcd import I2cLcd # imports I2cLcd class from pico_i2c_lcd
                                    # this was supplied by - https://www.tomshardware.com/how-to/lcd-display-raspberry-pi-pico
                                    # files saved to Pico - lcd_api.py | pico_i2c_lcd.py
import random # imports random module
import time # imports time module
import utime # imports utime module
                # honestly not sure if needed but some code examples I used utilized this so left it alone


######## SECTION 2 of 7: Assigning GPIO Pins to Hardware (global scope) ########

# Pin assignments below reference GPIO NUMBERS (RasPi Pico) and NOT the physical pins of the board itself
    # (MicroPython default for Pin class from machine module; functionality also exists to alter if wanted)
    
led_yR = Pin(0, Pin.OUT) # GP(IO)0 (p. 1) assigned to Red (RGB) LED as OUTput; Gate output(Y) LOW
led_yG = Pin(1, Pin.OUT) # GP(IO)1 (p. 2) assigned to Green (RGB) LED as OUTput; Gate output(Y) HIGH

i2c = I2C(0, sda = Pin(4), scl = Pin(5), freq = 400000) # GP(IO)4 & 5 (p. 6 & 7) assigned to I2C (i2c)
                                                            # equipped display for needed Data (sda) &
                                                            # Clock (scl) function w/ Frequency (freq) set
                                                            # to 400 kHz (400000 Hz)
                                                        # these are default values automatically constructed
                                                        # if creation of variable as: i2c = I2C(0) but it
                                                        # doesn't work properly if assigned as such b/c of
                                                        # utilizing 3rd party I2C driver (L. 36-38) that
                                                        # does not function unless explicitely assigning
                                                        # values within I2C parantheses

pb_an2 = Pin(6, Pin.IN, Pin.PULL_DOWN) # GP(IO)6 (p. 9) assigned to 2inAND (an2) PB as INput with PULL DOWN
                                        # enabled (incorporated to RasPi Pico)
pb_an3 = Pin(7, Pin.IN, Pin.PULL_DOWN) # GP(IO)7 (p. 10) assigned to 3inAND (an3) PB as INput  with PULL
                                        # DOWN enabled (incorporated to RasPi Pico)
pb_na2 = Pin(8, Pin.IN, Pin.PULL_DOWN) # GP(IO)8 (p. 11) assigned to 2inNAND (na2) PB as INput with PULL
                                        # DOWN enabled (incorporated to RasPi Pico)
pb_na3 = Pin(9, Pin.IN, Pin.PULL_DOWN) # GP(IO)9 (p. 12) assigned to 3inNAND (na3) PB as INput with PULL
                                        # DOWN enabled (incorporated to RasPi Pico)
pb_no2 = Pin(10, Pin.IN, Pin.PULL_DOWN) # GP(IO)10 (p. 14) assigned to 2inNOR (no2) PB as INput with PULL
                                        # DOWN enabled (incorporated to RasPi Pico)
pb_no3 = Pin(11, Pin.IN, Pin.PULL_DOWN) # GP(IO)11 (p. 15) assigned to 3inNOR (no3) PB as INput with PULL
                                        # DOWN enabled (incorporated to RasPi Pico)
pb_or2 = Pin(12, Pin.IN, Pin.PULL_DOWN) # GP(IO)12 (p. 16) assigned to 2inOR (or2) PB as INput with PULL
                                        # DOWN enabled (incorporated to RasPi Pico)
pb_or3 = Pin(13, Pin.IN, Pin.PULL_DOWN) # GP(IO)13 (p. 17) assigned to 3inOR (or3) PB as INput with PULL
                                        # DOWN enabled (incorporated to RasPi Pico)
pb_xo2 = Pin(14, Pin.IN, Pin.PULL_DOWN) # GP(IO)14 (p. 19) assigned to 2inXOR (xo3) PB as INput with PULL
                                        # DOWN enabled (incorporated to RasPi Pico)
pb_xn2 = Pin(15, Pin.IN, Pin.PULL_DOWN) # GP(IO)15 (p. 20) assigned to 2inXNOR (xn3) PB as INput with PULL
                                        # DOWN enabled (incorporated to RasPi Pico)

led_aR = Pin(16, Pin.OUT) # GP(IO)16 (p. 21) assigned to Red (RGB) LED as OUTput; Gate input(A) LOW
led_aG = Pin(17, Pin.OUT) # GP(IO)17 (p. 22) assigned to Green (RGB) LED as OUTput; Gate input(A) HIGH
led_bR = Pin(18, Pin.OUT) # GP(IO)18 (p. 24) assigned to Red (RGB) LED as OUTput; Gate input(B) LOW
led_bG = Pin(19, Pin.OUT) # GP(IO)19 (p. 25) assigned to Green (RGB) LED as OUTput; Gate input(B) HIGH
led_cR = Pin(20, Pin.OUT) # GP(IO)20 (p. 26) assigned to Red (RGB) LED as OUTput; Gate input(C) LOW
led_cG = Pin(21, Pin.OUT) # GP(IO)21 (p. 27) assigned to Green (RGB) LED as OUTput; Gate input(C) HIGH

pb_aIn = Pin(22, Pin.IN, Pin.PULL_DOWN) # GP(IO)22 (p. 29) assigned to input(A) (aIn) PB as INput; input(A)
                                            # toggle button with PULL DOWN enabled (incorporated to RasPi
                                            # Pico)
pb_bIn = Pin(26, Pin.IN, Pin.PULL_DOWN) # GP(IO)26 (p. 31) assigned to input(B) (bIn) PB as INput; input(B)
                                            # toggle button with PULL DOWN enabled (incorporated to RasPi
                                            # Pico)
pb_cIn = Pin(27, Pin.IN, Pin.PULL_DOWN) # GP(IO)27 (p. 32) assigned to input(C) (cIn) PB as INput; input(C)
                                            # toggle button with PULL DOWN enabled (incorporated to RasPi
                                            # Pico)

pb_res = Pin(28, Pin.IN, Pin.PULL_DOWN) # GP(IO)28 (p. 34) assigned to reset (res) PB as INput; reset game
                                            # button with PULL DOWN enabled (incorporated to RasPi Pico)


######## SECTION 3 of 7: Assigning (global) Variables ########

I2C_ADDR = i2c.scan()[0] # I2C_ADDR assigned to i2c.scan function
                            # this function came with 3rd party I2C driver
lcd = I2cLcd(i2c, I2C_ADDR, 2, 16) # lcd assigned to I2cLcd function
                                    # this function came with 3rd party I2C driver
                                    
lastTogg_a = time.ticks_ms() # lastTogg_a assigned to millisecond (ms) timer; for input(A) toggle
lastTogg_b = time.ticks_ms() # lastTogg_b assigned to millisecond (ms) timer; for input(B) toggle
lastTogg_c = time.ticks_ms() # lastTogg_c assigned to millisecond (ms) timer; for input(C) toggle

a = 0 # a assigned to (starting) 0; for Gate input logic levels
b = 0 # b assigned to (starting) 0; for Gate input logic levels
c = 0 # c assigned to (starting) 0; for Gate input logic levels

levelNumber = 0 # levelNumber assigned to (starting) 0; for level (stage) progeression from 0-9 (x10 steps)
stageScore = 14 # stageScore assigned to (starting) 14; for tracking possible stage points p/ level
totalScore = 140 # totalScore assigned to (starting) 140; for tracking total possible game points
finalPercent = 0 # finalPercent assigned to (starting) 0; for displaying point percentage upon completion
fRandom = random.randint(1, 2) #

playLevels = ['2inAND','3inAND','2inNAND','3inNAND','2inNOR','3inNOR','2inOR','3inOR','2inXOR','2inXNOR']
    # playLevels assigned list [...,...,...] value containing x10 entries (0-9 indexes) of Gate types;
        # for referencing to load specific Gate logic, determine in/correct Gate selection, and display text


######## SECTION 4 of 7: Defining Functions (global scope) ########
    
def and2(a, b): # function for a 2-input AND Logic Gate | a * b = y
    if a == 1 and b == 1:
        return True # **TRUE**: a and b = 1
    else:
        return False # FALSE: a or b = 0
    
def and3(a, b, c): # function for a 3-input AND Logic Gate | a * b * c = y
    if a == 1 and b == 1 and c == 1:
        return True # **TRUE**: a, b, and c = 1
    else:
        return False # FALSE: a, b, or c = 0
    
def nand2(a, b): # function for a 2-input NAND Logic Gate | (a * b)' = y
    if a == 1 and b == 1:
        return False # FALSE: a and b = 1
    else:
        return True # **TRUE**: a or b = 0
    
def nand3(a, b, c): # function for a 3-input NAND Logic Gate | (a * b * c)' = y
    if a == 1 and b == 1 and c == 1:
        return False # FALSE: a, b, and c = 1
    else:
        return True # **TRUE**: a, b, or c = 0
    
def nor2(a, b): # function for a 2-input NOR Logic Gate | (a + b)' = y
    if a == 0 and b == 0:
        return True # **TRUE**: a and b = 0
    else:
        return False # FALSE: a or b = 1
    
def nor3(a, b, c): # function for a 3-input NOR Logic Gate | (a + b + c)' = y
    if a == 0 and b == 0 and c == 0:
        return True # **TRUE**: a, b, and c = 0
    else:
        return False # FALSE: a, b, or c = 1
    
def or2(a, b): # function for a 2-input OR Logic Gate | a + b = y
    if a == 0 and b == 0:
        return False # FALSE: a and b = 0
    else:
        return True # **TRUE**: a, b, or c = 1
    
def or3(a, b, c): # function for a 3-input OR Logic Gate | a + b + c = y
    if a == 0 and b == 0 and c == 0:
        return False # FALSE: a, b, and c = 0
    else:
        return True # **TRUE**: a, b, or c = 1
    
def xor2(a, b): # function for a 2-input XOR Logic Gate | a (+) b = y
    if a != b:
        return True # **TRUE**: a = 0 and b = 1; a = 1 and b = 0
    else:
        return False # FALSE: a = 0 and b = 0; a = 1 and b = 1
    
def xnor2(a, b): # function for a 2-input XNOR Logic Gate | (a (+) b)' = y
    if a == b:
        return True # **TRUE**: a = 0 and b = 0; a = 1 and b = 1
    else:
        return False # FALSE: a = 0 and b = 1; a = 1 and b = 0

def gateCall(ID): # function for returning loaded Gate logic (works from playLevels list)
    if ID == '2inAND':
        return and2(a, b)
    elif ID == '3inAND':
        return and3(a, b, c)
    elif ID == '2inNAND':
        return nand2(a, b)
    elif ID == '3inNAND':
        return nand3(a, b, c)
    elif ID == '2inNOR':
        return nor2(a, b)
    elif ID == '3inNOR':
        return nor3(a, b, c)
    elif ID == '2inOR':
        return or2(a, b)
    elif ID == '3inOR':
        return or3(a, b, c)
    elif ID == '2inXOR':
        return xor2(a, b)
    elif ID == '2inXNOR':
        return xnor2(a, b)

def inpStart(argID): # function for setting input LEDs from beginning of new stage / Gate (works from
                        # playLevels list) and utilizes a Pin class functionality of "xxx.value()"
                        # where 1 = HIGH and 0 = LOW
    if argID == '2inAND' or argID == '2inNAND' or argID == '2inNOR' or argID == '2inOR' or argID == '2inXOR' or argID == '2inXNOR':
        led_aR.value(1)
        led_aG.value(0)
        led_bR.value(1)
        led_bG.value(0)
        led_cR.value(0) # because only 2-inputs, (C) is "off"
        led_cG.value(0) # ' ' (above)
        cNull() # calls cNull function for "off" c (variable)
    elif argID == '3inAND' or argID == '3inNAND' or argID == '3inNOR' or argID == '3inOR':
        led_aR.value(1)
        led_aG.value(0)
        led_bR.value(1)
        led_bG.value(0)
        led_cR.value(1)
        led_cG.value(0)

def cNull(): # function for assigning (variable) c as 'N/A' in off condition (only 2-input Gate)
    global c # global used because need to reference variable in global scope (outside game loop)
    c = 'N/A'

def outSetter(logic): # function for setting output LED based off of logic for loaded Gate
                        # utilizes a Pin class functionality of "xxx.value()" where 1 = HIGH and 0 = LOW
    if logic == True:
        led_yR.value(0)
        led_yG.value(1)
    else:
        led_yR.value(1)
        led_yG.value(0)

def inpReset(): # function for resetting Gate input values to 0 (after each stage because need to start LOW)
    global a, b, c # global used because need to reference variables in global scope (outside game loop)
    a = 0
    b = 0
    c = 0

def scoreReset(): # function for resetting stageScore to 0 (after each stage because need to start LOW)
    global stageScore # global used because need to reference variable in global scope (outside game loop)
    stageScore = 14

def scoreDeduct(): # function for deducting stageScore from totalScore (if wrong Gate selected)
    global stageScore, totalScore # global used because need to reference variable in global scope
                                    # (outside game loop)
    totalScore = totalScore - stageScore

def scorePercent(): # function for converting totalScore to a percentage (for game end)
    global finalPercent, totalScore # global used because need to reference variable in global scope
                                        # (outside game loop)
    finalPercent = str(round(float((totalScore / 140)*100),1)) # assigns finalPercent variable as a string
                                                                # of the rounded (to 1 decimal) floating
                                                                # number result of (totalScore / 140)*100;
                                                                # the ",1" near the end of the parantheses
                                                                # tells *round* to go to 1 decimal place

def playDisp(): # function for using I2C display to show text typical during stages / playthrough
    lcd.clear() # clears display screen (precoded function from 3rd party I2C driver)
                    # needed otherwise text will be "jumbled" with what was displayed previously
    lcd.putstr('NAME!THAT!GATE! \n'
               'Score: '+ str(totalScore) +'') # displays string of totalScore variable
        
def cNotValDisp(): # function for using I2C display to show text if (C) toggled but variable c = 'N/A'
    lcd.clear() # clears display screen (precoded function from 3rd party I2C driver)
                    # needed otherwise text will be "jumbled" with what was displayed previously
    lcd.putstr('INP(C) NOT VALID\n'
               'Score: '+ str(totalScore) +'') # displays string of totalScore variable

def rightDisp(): # function for using I2C display to show text if Gate selected is right one
    lcd.clear() # clears display screen (precoded function from 3rd party I2C driver)
                    # needed otherwise text will be "jumbled" with what was displayed previously
    lcd.putstr('Correct!!       \n'
               'Gate: '+ playLevels[levelNumber] +'') # displays Gate loaded (already a string value) by
                                                        # using levelNumber as argument to call the indexed
                                                        # value contained in playLevels list (i.e. if
                                                        # levelNumber = 0 then calls list value at index 0
                                                        # of playLevels, levelNumber = 9 then index 9
                                                        # called, etc.)

def wrongDisp(): # function for using I2C display to show text if Gate selected is wrong one
    lcd.clear() # clears display screen (precoded function from 3rd party I2C driver)
                    # needed otherwise text will be "jumbled" with what was displayed previously
    lcd.putstr('Incorrect..     \n'
               'Gate: '+ playLevels[levelNumber] +'') # displays Gate loaded (already a string value) by
                                                        # using levelNumber as argument to call the indexed
                                                        # value contained in playLevels list (i.e. if
                                                        # levelNumber = 0 then calls list value at index 0
                                                        # of playLevels, levelNumber = 9 then index 9
                                                        # called, etc.)
    
def endDisp(): # function for using I2C display to show *score* text when game / playthrough has ended
    lcd.clear() # clears display screen (precoded function from 3rd party I2C driver)
                    # needed otherwise text will be "jumbled" with what was displayed previously
    lcd.putstr('Finished! '+ str(finalPercent) +'%\n' # displays string of finalPercent
               'Final Score: '+ str(totalScore) +'') # displays string of totalScore (final)

def funDisp(): # function for using I2C display to show *fun* text when game / playthrough has ended
                # NOTE: there are spaces at end of some strings which seem to help text from "jumbling"
    global totalScore, fRandom # global used because need to reference variable in global scope
    if totalScore >= 134: # essentially 134-140 (95.7-100%) b/c fairly certain can't get w/o lucky guess
        lcd.clear() # clears display screen (precoded function from 3rd party I2C driver)
        lcd.putstr('Wise AND lucky!?\n') # no reference; just thought summed up situation if arrived here
        
    elif totalScore >= 132: # essentially 132-133 (94.3-95%)
        lcd.clear() # clears display screen (precoded function from 3rd party I2C driver)
        lcd.putstr('Go build a dam!!\n') # M. Beavers reference; Michael is a digital man living in an
                                            # analog world afterall and his courses have taught me so much
                                            # of the logic and discipline behind this project as well as
                                            # how to become a better critical thinker.        
    elif totalScore >= 126: # essentially 126-131 (90-93.6%)
        lcd.clear() # clears display screen (precoded function from 3rd party I2C driver)
        lcd.putstr('Not just a land \n' # J. Tillman reference; Joes' effort into providing his students w/
                   'tiller huh?\n')         # everything they might need, from academics, laughter, &
                                            # words of wisdom / encouragement, is profound and I'm a better
                                            # student and person from the opportunity of having him as an
                                            # instructor.        
    elif totalScore >= 112: # essentially 112-125 (80-89.3%)
        lcd.clear() # clears display screen (precoded function from 3rd party I2C driver)
        lcd.putstr('We can spare the\n' # S. Rhine reference; Scott was only my instructor once (Innovation)
                   'ID-10-T error\n')       # and he has such an obvious & contagious passion for being a
                                            # "maker" that it inspired me greatly to forge ahead on my
                                            # path to become more technologically knowledgeable & creative.        
    elif totalScore >= 98: # essentially 98-111 (70-79.3%)
        lcd.clear() # clears display screen (precoded function from 3rd party I2C driver)
        lcd.putstr('Better lay off  \n' # G. Lindley reference; Gary taught me Applied AC/DC Circuits
                   'the markers bud\n')     # which were my first of many courses I've had towards App. Eng.
                                            # and it was there then that he noticed & convinced me I would
                                            # be a good fit for the program so w/o him I very likely would.
                                            # not be where and as satisfied as I am today.
    else: # essentially 0-97 (0-69.3%)
        if fRandom == 1: # when (global) fRandom variable is 1 (is assigned random value between 1 & 2)
            lcd.clear() # clears display screen (precoded function from 3rd party I2C driver)
            lcd.putstr('Out of your     \n' # L. Kitten reference; Leo and I didn't have a lot of
                       'element kitty!\n')      # interaction together (only taught once for 3D Modeling)
                                                # but I think he is a pretty cool dude who also helped me
                                                # learn essential concepts and powerful software for
                                                # creating which I have a very strong interest in.
        elif fRandom == 2: # when (global) fRandom variable is 2 (is assigned random value between 1 & 2)
            lcd.clear() # clears display screen (precoded function from 3rd party I2C driver)
            lcd.putstr('Buzz off!       \n') # D. Buzzard reference; Dion and I also didn't have a lot of
                                                # interaction together (only taught once for Technical
                                                # Mechanisms) but I think very highly of him for always
                                                # providing me with his helpful assistance / opinion
                                                # and also just being a very kind and fun guy to be around.

def rightFlash(): # function for flashing LEDs green 4 times in 2s / 2000ms (if Gate selection right)
                    # utilizes a Pin class functionality of "xxx.value()" where 1 = HIGH and 0 = LOW
                    # and "xxx.toggle()" which switches from previous state to complement (1 -> 0, 0 -> 1)
    for i in range(1): # for only (1) time, sets all LEDs to green (starting)
        led_aR.value(0)
        led_aG.value(1)
        led_bR.value(0)
        led_bG.value(1)
        led_cR.value(0)
        led_cG.value(1)
        led_yR.value(0)
        led_yG.value(1)
        time.sleep(0.25) # holds ("sleeps") w/ green on for 1/4 of a second (250ms)
    for i in range(7): # for a loop of only (7) times, toggles green LED off -> on, on -> off, ...
        led_aG.toggle()
        led_bG.toggle()
        led_cG.toggle()
        led_yG.toggle()
        time.sleep(0.25) # holds ("sleeps") each toggle for 1/4 of a second (250ms)

def wrongFlash(): # function for flashing LEDs red 4 times in 2s / 2000ms (if Gate selection wrong)
                    # utilizes a Pin class functionality of "xxx.value()" where 1 = HIGH and 0 = LOW
                    # and "xxx.toggle()" which switches from previous state to complement (1 -> 0, 0 -> 1)
    for i in range(1): # for only (1) time, sets all LEDs to red (starting)
        led_aR.value(1)
        led_aG.value(0)
        led_bR.value(1)
        led_bG.value(0)
        led_cR.value(1)
        led_cG.value(0)
        led_yR.value(1)
        led_yG.value(0)
        time.sleep(0.25) # holds ("sleeps") w/ green on for 1/4 of a second (250ms)
    for i in range(7): # for a loop of only (7) times, toggles green LED off -> on, on -> off, ...
        led_aR.toggle()
        led_bR.toggle()
        led_cR.toggle()
        led_yR.toggle()
        time.sleep(0.25) # holds ("sleeps") each toggle for 1/4 of a second (250ms)

def inpEnd(): # function for setting LEDs to staggered (RGRG) pattern at game end
                # utilizes a Pin class functionality of "xxx.value()" where 1 = HIGH and 0 = LOW
    led_aR.value(1)
    led_aG.value(0)
    led_bR.value(0)
    led_bG.value(1)
    led_cR.value(1)
    led_cG.value(0)
    led_yR.value(0)
    led_yG.value(1)

def endTogg(): # function for toggling LEDs in staggered (RGRG -> GRGR, ...) pattern at game end
                # utilizes a Pin class functionality of "xxx.toggle()" which switches from previous state
                # to complement (1 -> 0, 0 -> 1)
    led_aR.toggle()
    led_aG.toggle()
    led_bR.toggle()
    led_bG.toggle()
    led_cR.toggle()
    led_cG.toggle()
    led_yR.toggle()
    led_yG.toggle()

def inpToggler(inpButton): # function for toggling input variables (a, b, c) w/ respective PBs and is the
                            # "handler" used in conjunction with Pin class functionality "xxx.irq()" which
                            # is an "interrupt" and essentially will trigger this event when activated
    global a, b, c, lastTogg_a, lastTogg_b, lastTogg_c, pb_aIn, pb_bIn, pb_cIn, notValid_c, stageScore, totalScore
        # global used because need to reference variables in global scope
    if inpButton is pb_aIn and a == 0: # when PB for Gate input (A) is activated and variable a = 0
        if time.ticks_diff(time.ticks_ms(), lastTogg_a) > 500: # when difference between ms ticks (starts at
                                                                # initialization and restarts after every
                                                                # inpToggler activation) is greater than
                                                                # lastTogg_a by 0.5s (500ms)
            if stageScore != 0: # when stageScore does not equal 0
                a = a + 1 # variable a increases by 1 (0 + 1 = 1)
                led_aR.toggle() # toggles red LED for input (A) from previous value (off -> on, on -> off)
                led_aG.toggle() # toggles green LED for input (A) from previous value (off -> on, on -> off)
                outSetter(gateCall(playLevels[levelNumber])) # calls outSetter function output (Y) condition
                                                                # passes gateCall function to with argument
                                                                # of index value in playLevels list equal to
                                                                # levelNumber (0 = index 0, 9 = index 9, etc.)
                stageScore = stageScore - 1 # variable stageScore decreases by 1 (stageScore - 1)
                totalScore = totalScore - 1 # variable totalScore decreases by 1 (totalScore - 1)
                playDisp() # calls playDisp function (shows updated totalScore)
                lastTogg_a = time.ticks_ms() # variable lastTogg_a restarts ticks ("counting") in ms
                lastTogg_b = time.ticks_ms() # variable lastTogg_b restarts ticks ("counting") in ms
                lastTogg_c = time.ticks_ms() # variable lastTogg_c restarts ticks ("counting") in ms
            elif stageScore == 0: # when stageScore does equal 0
                a = a + 1 # variable a increases by 1 (0 + 1 = 1)
                led_aR.toggle() # toggles red LED for input (A) from previous value (off -> on, on -> off)
                led_aG.toggle() # toggles green LED for input (A) from previous value (off -> on, on -> off)
                outSetter(gateCall(playLevels[levelNumber])) # calls outSetter function output (Y) condition
                                                                # passes gateCall function to with argument
                                                                # of index value in playLevels list equal to
                                                                # levelNumber (0 = index 0, 9 = index 9, etc.)
                lastTogg_a = time.ticks_ms() # variable lastTogg_a restarts ticks ("counting") in ms
                lastTogg_b = time.ticks_ms() # variable lastTogg_b restarts ticks ("counting") in ms
                lastTogg_c = time.ticks_ms() # variable lastTogg_c restarts ticks ("counting") in ms
    elif inpButton is pb_aIn and a == 1: # when PB for Gate input (A) is activated and variable a = 1
        if time.ticks_diff(time.ticks_ms(), lastTogg_a) > 500: # when difference between ms ticks (starts at
                                                                # initialization and restarts after every
                                                                # inpToggler activation) is greater than
                                                                # lastTogg_a by 0.5s (500ms)
            if stageScore != 0: # when stageScore does not equal 0
                a = a - 1 # variable a decreases by 1 (1 - 1 = 0)
                led_aR.toggle() # toggles red LED for input (A) from previous value (off -> on, on -> off)
                led_aG.toggle() # toggles green LED for input (A) from previous value (off -> on, on -> off)
                outSetter(gateCall(playLevels[levelNumber])) # calls outSetter function output (Y) condition
                                                                # passes gateCall function to with argument
                                                                # of index value in playLevels list equal to
                                                                # levelNumber (0 = index 0, 9 = index 9, etc.)
                stageScore = stageScore - 1 # variable stageScore decreases by 1 (stageScore - 1)
                totalScore = totalScore - 1 # variable totalScore decreases by 1 (totalScore - 1)
                playDisp() # calls playDisp function (shows updated totalScore)
                lastTogg_a = time.ticks_ms() # variable lastTogg_a restarts ticks ("counting") in ms
                lastTogg_b = time.ticks_ms() # variable lastTogg_b restarts ticks ("counting") in ms
                lastTogg_c = time.ticks_ms() # variable lastTogg_c restarts ticks ("counting") in ms
            elif stageScore == 0: # when stageScore does equal 0
                a = a - 1 # variable a decreases by 1 (1 - 1 = 0)
                led_aR.toggle() # toggles red LED for input (A) from previous value (off -> on, on -> off)
                led_aG.toggle() # toggles green LED for input (A) from previous value (off -> on, on -> off)
                outSetter(gateCall(playLevels[levelNumber])) # calls outSetter function output (Y) condition
                                                                # passes gateCall function to with argument
                                                                # of index value in playLevels list equal to
                                                                # levelNumber (0 = index 0, 9 = index 9, etc.)
                lastTogg_a = time.ticks_ms() # variable lastTogg_a restarts ticks ("counting") in ms
                lastTogg_b = time.ticks_ms() # variable lastTogg_b restarts ticks ("counting") in ms
                lastTogg_c = time.ticks_ms() # variable lastTogg_c restarts ticks ("counting") in ms
    elif inpButton is pb_bIn and b == 0: # when PB for Gate input (B) is activated and variable b = 0
        if time.ticks_diff(time.ticks_ms(), lastTogg_b) > 500: # when difference between ms ticks (starts at
                                                                # initialization and restarts after every
                                                                # inpToggler activation) is greater than
                                                                # lastTogg_b by 0.5s (500ms)
            if stageScore != 0: # when stageScore does not equal 0
                b = b + 1 # variable b increases by 1 (0 + 1 = 1)
                led_bR.toggle() # toggles red LED for input (B) from previous value (off -> on, on -> off)
                led_bG.toggle() # toggles green LED for input (B) from previous value (off -> on, on -> off)
                outSetter(gateCall(playLevels[levelNumber])) # calls outSetter function output (Y) condition
                                                                # passes gateCall function to with argument
                                                                # of index value in playLevels list equal to
                                                                # levelNumber (0 = index 0, 9 = index 9, etc.)
                stageScore = stageScore - 1 # variable stageScore decreases by 1 (stageScore - 1)
                totalScore = totalScore - 1 # variable totalScore decreases by 1 (totalScore - 1)
                playDisp() # calls playDisp function (shows updated totalScore)
                lastTogg_b = time.ticks_ms() # variable lastTogg_b restarts ticks ("counting") in ms
                lastTogg_c = time.ticks_ms() # variable lastTogg_c restarts ticks ("counting") in ms
                lastTogg_a = time.ticks_ms() # variable lastTogg_a restarts ticks ("counting") in ms
            elif stageScore == 0: # when stageScore does equal 0
                b = b + 1 # variable b increases by 1 (0 + 1 = 1)
                led_bR.toggle() # toggles red LED for input (B) from previous value (off -> on, on -> off)
                led_bG.toggle() # toggles green LED for input (B) from previous value (off -> on, on -> off)
                outSetter(gateCall(playLevels[levelNumber])) # calls outSetter function output (Y) condition
                                                                # passes gateCall function to with argument
                                                                # of index value in playLevels list equal to
                                                                # levelNumber (0 = index 0, 9 = index 9, etc.)
                lastTogg_b = time.ticks_ms() # variable lastTogg_b restarts ticks ("counting") in ms
                lastTogg_c = time.ticks_ms() # variable lastTogg_c restarts ticks ("counting") in ms
                lastTogg_a = time.ticks_ms() # variable lastTogg_a restarts ticks ("counting") in ms
    elif inpButton is pb_bIn and b == 1: # when PB for Gate input (B) is activated and variable b = 1
        if time.ticks_diff(time.ticks_ms(), lastTogg_b) > 500: # when difference between ms ticks (starts at
                                                                # initialization and restarts after every
                                                                # inpToggler activation) is greater than
                                                                # lastTogg_b by 0.5s (500ms)
            if stageScore != 0: # when stageScore does not equal 0
                b = b - 1 # variable b decreases by 1 (1 - 1 = 0)
                led_bR.toggle() # toggles red LED for input (B) from previous value (off -> on, on -> off)
                led_bG.toggle() # toggles green LED for input (B) from previous value (off -> on, on -> off)
                outSetter(gateCall(playLevels[levelNumber])) # calls outSetter function output (Y) condition
                                                                # passes gateCall function to with argument
                                                                # of index value in playLevels list equal to
                                                                # levelNumber (0 = index 0, 9 = index 9, etc.)
                stageScore = stageScore - 1 # variable stageScore decreases by 1 (stageScore - 1)
                totalScore = totalScore - 1 # variable totalScore decreases by 1 (totalScore - 1)
                playDisp() # calls playDisp function (shows updated totalScore)
                lastTogg_b = time.ticks_ms() # variable lastTogg_b restarts ticks ("counting") in ms
                lastTogg_c = time.ticks_ms() # variable lastTogg_c restarts ticks ("counting") in ms
                lastTogg_a = time.ticks_ms() # variable lastTogg_a restarts ticks ("counting") in ms
            elif stageScore == 0: # when stageScore does equal 0
                b = b - 1 # variable b decreases by 1 (1 - 1 = 0)
                led_bR.toggle() # toggles red LED for input (B) from previous value (off -> on, on -> off)
                led_bG.toggle() # toggles green LED for input (B) from previous value (off -> on, on -> off)
                outSetter(gateCall(playLevels[levelNumber])) # calls outSetter function output (Y) condition
                                                                # passes gateCall function to with argument
                                                                # of index value in playLevels list equal to
                                                                # levelNumber (0 = index 0, 9 = index 9, etc.)
                lastTogg_b = time.ticks_ms() # variable lastTogg_b restarts ticks ("counting") in ms
                lastTogg_c = time.ticks_ms() # variable lastTogg_c restarts ticks ("counting") in ms
                lastTogg_a = time.ticks_ms() # variable lastTogg_a restarts ticks ("counting") in ms
    elif inpButton is pb_cIn and c == 0: # when PB for Gate input (C) is activated and variable c = 0
        if time.ticks_diff(time.ticks_ms(), lastTogg_c) > 500: # when difference between ms ticks (starts at
                                                                # initialization and restarts after every
                                                                # inpToggler activation) is greater than
                                                                # lastTogg_c by 0.5s (500ms)
            if stageScore != 0: # when stageScore does not equal 0
                c = c + 1 # variable c increases by 1 (0 + 1 = 1)
                led_cR.toggle() # toggles red LED for input (C) from previous value (off -> on, on -> off)
                led_cG.toggle() # toggles green LED for input (C) from previous value (off -> on, on -> off)
                outSetter(gateCall(playLevels[levelNumber])) # calls outSetter function output (Y) condition
                                                                # passes gateCall function to with argument
                                                                # of index value in playLevels list equal to
                                                                # levelNumber (0 = index 0, 9 = index 9, etc.)
                stageScore = stageScore - 1 # variable stageScore decreases by 1 (stageScore - 1)
                totalScore = totalScore - 1 # variable totalScore decreases by 1 (totalScore - 1)
                playDisp() # calls playDisp function (shows updated totalScore)
                lastTogg_c = time.ticks_ms() # variable lastTogg_c restarts ticks ("counting") in ms
                lastTogg_a = time.ticks_ms() # variable lastTogg_a restarts ticks ("counting") in ms
                lastTogg_b = time.ticks_ms() # variable lastTogg_b restarts ticks ("counting") in ms
            elif stageScore == 0: # when stageScore does equal 0
                c = c + 1 # variable c increases by 1 (0 + 1 = 1)
                led_cR.toggle() # toggles red LED for input (C) from previous value (off -> on, on -> off)
                led_cG.toggle() # toggles green LED for input (C) from previous value (off -> on, on -> off)
                outSetter(gateCall(playLevels[levelNumber])) # calls outSetter function output (Y) condition
                                                                # passes gateCall function to with argument
                                                                # of index value in playLevels list equal to
                                                                # levelNumber (0 = index 0, 9 = index 9, etc.)
                lastTogg_c = time.ticks_ms() # variable lastTogg_c restarts ticks ("counting") in ms
                lastTogg_a = time.ticks_ms() # variable lastTogg_a restarts ticks ("counting") in ms
                lastTogg_b = time.ticks_ms() # variable lastTogg_b restarts ticks ("counting") in ms
    elif inpButton is pb_cIn and c == 1: # when PB for Gate input (C) is activated and variable c = 1
        if time.ticks_diff(time.ticks_ms(), lastTogg_c) > 500: # when difference between ms ticks (starts at
                                                                # initialization and restarts after every
                                                                # inpToggler activation) is greater than
                                                                # lastTogg_c by 0.5s (500ms)
            if stageScore != 0: # when stageScore does not equal 0
                c = c - 1 # variable c decreases by 1 (1 - 1 = 0)
                led_cR.toggle() # toggles red LED for input (C) from previous value (off -> on, on -> off)
                led_cG.toggle() # toggles green LED for input (C) from previous value (off -> on, on -> off)
                outSetter(gateCall(playLevels[levelNumber])) # calls outSetter function output (Y) condition
                                                                # passes gateCall function to with argument
                                                                # of index value in playLevels list equal to
                                                                # levelNumber (0 = index 0, 9 = index 9, etc.)
                stageScore = stageScore - 1 # variable stageScore decreases by 1 (stageScore - 1)
                totalScore = totalScore - 1 # variable totalScore decreases by 1 (totalScore - 1)
                playDisp() # calls playDisp function (shows updated totalScore)
                lastTogg_c = time.ticks_ms() # variable lastTogg_c restarts ticks ("counting") in ms
                lastTogg_a = time.ticks_ms() # variable lastTogg_a restarts ticks ("counting") in ms
                lastTogg_b = time.ticks_ms() # variable lastTogg_b restarts ticks ("counting") in ms
            elif stageScore == 0: # when stageScore does equal 0
                c = c - 1 # variable c decreases by 1 (1 - 1 = 0)
                led_cR.toggle() # toggles red LED for input (C) from previous value (off -> on, on -> off)
                led_cG.toggle() # toggles green LED for input (C) from previous value (off -> on, on -> off)
                outSetter(gateCall(playLevels[levelNumber])) # calls outSetter function output (Y) condition
                                                                # passes gateCall function to with argument
                                                                # of index value in playLevels list equal to
                                                                # levelNumber (0 = index 0, 9 = index 9, etc.)
                lastTogg_c = time.ticks_ms() # variable lastTogg_c restarts ticks ("counting") in ms
                lastTogg_a = time.ticks_ms() # variable lastTogg_a restarts ticks ("counting") in ms
                lastTogg_b = time.ticks_ms() # variable lastTogg_b restarts ticks ("counting") in ms
    elif inpButton is pb_cIn and c == 'N/A': # when PB for Gate input (C) is activated and variable c = 'N/A'
        if time.ticks_diff(time.ticks_ms(), lastTogg_c) > 500: # when difference between ms ticks (starts at
                                                                # initialization and restarts after every
                                                                # inpToggler activation) is greater than
                                                                # lastTogg_c by 0.5s (500ms)
            if stageScore != 0: # when stageScore does not equal 0
                stageScore = stageScore - 1 # variable stageScore decreases by 1 (stageScore - 1)
                totalScore = totalScore - 1 # variable totalScore decreases by 1 (totalScore - 1)
                cNotValDisp() # calls cNotValDisp function (tells input (C) not valid)
                time.sleep(2) # holds ("sleeps") w/ cNotValDisp for 2s (2000ms) so display readable b/c
                                # would just flash on for split second otherwise
                playDisp() # calls playDisp function (shows updated totalScore)
                lastTogg_c = time.ticks_ms() # variable lastTogg_c restarts ticks ("counting") in ms
                lastTogg_a = time.ticks_ms() # variable lastTogg_a restarts ticks ("counting") in ms
                lastTogg_b = time.ticks_ms() # variable lastTogg_b restarts ticks ("counting") in ms
            elif stageScore == 0: # when stageScore does equal 0
                cNotValDisp() # calls cNotValDisp function (tells input (C) not valid)
                time.sleep(2) # holds ("sleeps") w/ cNotValDisp for 2s (2000ms) so display readable b/c
                                # would just flash on for split second otherwise
                lastTogg_c = time.ticks_ms() # variable lastTogg_c restarts ticks ("counting") in ms
                lastTogg_a = time.ticks_ms() # variable lastTogg_a restarts ticks ("counting") in ms
                lastTogg_b = time.ticks_ms() # variable lastTogg_b restarts ticks ("counting") in ms
    elif inpButton is pb_res: # when PB for reset is activated
        machine.reset() # machine module functionality for resetting (rebooting) device from start
        
def randomShuffle(seq): # function for a randomized shuffle b/c MicroPython does not include random.shuffle
                            # from random module like standard Python
    l = len(seq) # variable l assigned as the length ( 0, 1, 2, ...) of values contained in seq which is the
                    # argument passed to the randomShuffle function
    for i in range(l): # for a loop the amount of times equal to l
        j = random.randrange(l) # variable j assigned as a randomly selected element from the range
                                    # (randrange; from random module) of the argument passed which is l
        seq[i], seq[j] = seq[j], seq[i] # for what i is (from in range loop) referenced to (randrange)
                                            # variable j, the variable j becomes the "indexed" value at
                                            # position i (this was example code I used so may not be exact /
                                            # correct explanation)


######## SECTION 5 of 7: Setting Up for Start of Game ########
                                            
randomShuffle(playLevels) # calls randomShuffle function to randomize indexed values in playLevels list
playDisp() # calls playDisp function (shows starting totalScore)


######## SECTION 6 of 7: Game Loop ########

for i in range(10): # for a loop of only (10) times, game is active (because only x10 Gates / stages)
    
    pb_aIn.irq(trigger = Pin.IRQ_RISING, handler = inpToggler) # designates pb.aIn to work as an "interrupt"
                                                                # (.irq) which is a Pin class functionality;
                                                                # when triggered (on RISING edge) will look
                                                                # to handler inpToggler (function) for
                                                                # instruction on what to execute
    pb_bIn.irq(trigger = Pin.IRQ_RISING, handler = inpToggler)# designates pb.bIn to work as an "interrupt"
                                                                # (.irq) which is a Pin class functionality;
                                                                # when triggered (on RISING edge) will look
                                                                # to handler inpToggler (function) for
                                                                # instruction on what to execute
    pb_cIn.irq(trigger = Pin.IRQ_RISING, handler = inpToggler)# designates pb.cIn to work as an "interrupt"
                                                                # (.irq) which is a Pin class functionality;
                                                                # when triggered (on RISING edge) will look
                                                                # to handler inpToggler (function) for
                                                                # instruction on what to execute
    pb_res.irq(trigger = Pin.IRQ_FALLING, handler = inpToggler)# designates pb.res to work as an "interrupt"
                                                                # (.irq) which is a Pin class functionality;
                                                                # when triggered (on FALLING edge) will look
                                                                # to handler inpToggler (function) for
                                                                # instruction on what to execute
    
    inpStart(playLevels[levelNumber]) # calls function inpStart with argument being playLevels (list) w/ the
                                        # index value equal to levelNumber (0 = index 0, 9 = index 9, etc.);
                                        # essentially sets the input LEDs that indicate logic level which
                                        # always start LOW but, depending on number of inputs loaded Gate
                                        # will have, input(C) will either be on or off
    outSetter(gateCall(playLevels[levelNumber])) # calls function outSetter with argument being what gateCall
                                                    # returns with its' own argument of the playLevels (list)
                                                    # w/ the index value equal to levelNumber (0 = index 0,
                                                    # 9 = index 9, etc.); essentially sets the output LED
                                                    # based on the logic of the input variables (a, b, or c)
    gateLoaded = playLevels[levelNumber] # gateLoaded assigned to playLevels (list) w/ the index value equal
                                            # to levelNumber (0 = index 0, 9 = index 9, etc.); essentially
                                            # sets up referenced variable as the loaded Gate name for the
                                            # following if condition block that dictates in/correct Gate
                                            # selection and respective results contained
    
    if gateLoaded == '2inAND': # if gateLoaded is 2inAND
        while True: # infinite loop until broken (break statement)
            if pb_an2.value() == True: # if pb_an2 (2inAND) selected
                rightDisp() # calls rightDisp function (displays right choice selected)
                rightFlash() # calls rightFlash function (all LEDs flash green for right choice)
                break # infinite loop broken
            elif pb_an3.value() == True or pb_na2.value() == True or pb_na3.value() == True or pb_no2.value() == True or pb_no3.value() == True or pb_or2.value() == True or pb_or3.value() == True or pb_xo2.value() == True or pb_xn2.value() == True:
                    # if pb_an3/pb_na2/pb_na3/pb_no2/pb_no3/pb_or2/pb_or3/pb_xo2/pb_xn2 selected
                wrongDisp() # calls wrongDisp function (displays wrong choice selected)
                wrongFlash() # calls wrongFlash function (all LEDs flash red for right choice)
                scoreDeduct() # calls scoreDeduct function (subtracts stageScore from totalScore)
                break # infinite loop broken
    elif gateLoaded == '3inAND': # if gateLoaded is 3inAND
        while True: # infinite loop until broken (break statement)
            if pb_an3.value() == True: # if pb_an3 (3inAND) selected
                rightDisp() # calls rightDisp function (displays right choice selected)
                rightFlash() # calls rightFlash function (all LEDs flash green for right choice)
                break # infinite loop broken
            elif pb_an2.value() == True or pb_na2.value() == True or pb_na3.value() == True or pb_no2.value() == True or pb_no3.value() == True or pb_or2.value() == True or pb_or3.value() == True or pb_xo2.value() == True or pb_xn2.value() == True:
                    # if pb_an2/pb_na2/pb_na3/pb_no2/pb_no3/pb_or2/pb_or3/pb_xo2/pb_xn2 selected
                wrongDisp() # calls wrongDisp function (displays wrong choice selected)
                wrongFlash() # calls wrongFlash function (all LEDs flash red for right choice)
                scoreDeduct() # calls scoreDeduct function (subtracts stageScore from totalScore)
                break # infinite loop broken
    elif gateLoaded == '2inNAND': # if gateLoaded is 2inNAND
        while True: # infinite loop until broken (break statement)
            if pb_na2.value() == True: # if pb_na2 (2inNAND) selected
                rightDisp() # calls rightDisp function (displays right choice selected)
                rightFlash() # calls rightFlash function (all LEDs flash green for right choice)
                break # infinite loop broken
            elif pb_an2.value() == True or pb_an3.value() == True or pb_na3.value() == True or pb_no2.value() == True or pb_no3.value() == True or pb_or2.value() == True or pb_or3.value() == True or pb_xo2.value() == True or pb_xn2.value() == True:
                    # if pb_an2/pb_an3/pb_na3/pb_no2/pb_no3/pb_or2/pb_or3/pb_xo2/pb_xn2 selected
                wrongDisp() # calls wrongDisp function (displays wrong choice selected)
                wrongFlash() # calls wrongFlash function (all LEDs flash red for right choice)
                scoreDeduct() # calls scoreDeduct function (subtracts stageScore from totalScore)
                break # infinite loop broken
    elif gateLoaded == '3inNAND': # if gateLoaded is 3inNAND
        while True: # infinite loop until broken (break statement)
            if pb_na3.value() == True: # if pb_na3 (3inNAND) selected
                rightDisp() # calls rightDisp function (displays right choice selected)
                rightFlash() # calls rightFlash function (all LEDs flash green for right choice)
                break # infinite loop broken
            elif pb_an2.value() == True or pb_an3.value() == True or pb_na2.value() == True or pb_no2.value() == True or pb_no3.value() == True or pb_or2.value() == True or pb_or3.value() == True or pb_xo2.value() == True or pb_xn2.value() == True:
                    # if pb_an2/pb_an3/pb_na2/pb_no2/pb_no3/pb_or2/pb_or3/pb_xo2/pb_xn2 selected
                wrongDisp() # calls wrongDisp function (displays wrong choice selected)
                wrongFlash() # calls wrongFlash function (all LEDs flash red for right choice)
                scoreDeduct() # calls scoreDeduct function (subtracts stageScore from totalScore)
                break # infinite loop broken
    elif gateLoaded == '2inNOR': # if gateLoaded is 2inNOR
        while True: # infinite loop until broken (break statement)
            if pb_no2.value() == True: # if pb_no2 (2inNOR) selected
                rightDisp() # calls rightDisp function (displays right choice selected)
                rightFlash() # calls rightFlash function (all LEDs flash green for right choice)
                break # infinite loop broken
            elif pb_an2.value() == True or pb_an3.value() == True or pb_na2.value() == True or pb_na3.value() == True or pb_no3.value() == True or pb_or2.value() == True or pb_or3.value() == True or pb_xo2.value() == True or pb_xn2.value() == True:
                    # if pb_an2/pb_an3/pb_na2/pb_na3/pb_no3/pb_or2/pb_or3/pb_xo2/pb_xn2 selected
                wrongDisp() # calls wrongDisp function (displays wrong choice selected)
                wrongFlash() # calls wrongFlash function (all LEDs flash red for right choice)
                scoreDeduct() # calls scoreDeduct function (subtracts stageScore from totalScore)
                break # infinite loop broken
    elif gateLoaded == '3inNOR': # if gateLoaded is 3inNOR
        while True: # infinite loop until broken (break statement)
            if pb_no3.value() == True: # if pb_no3 (3inNOR) selected
                rightDisp() # calls rightDisp function (displays right choice selected)
                rightFlash() # calls rightFlash function (all LEDs flash green for right choice)
                break # infinite loop broken
            elif pb_an2.value() == True or pb_an3.value() == True or pb_na2.value() == True or pb_na3.value() == True or pb_no2.value() == True or pb_or2.value() == True or pb_or3.value() == True or pb_xo2.value() == True or pb_xn2.value() == True:
                    # if pb_an2/pb_an3/pb_na2/pb_na3/pb_no2/pb_or2/pb_or3/pb_xo2/pb_xn2 selected
                wrongDisp() # calls wrongDisp function (displays wrong choice selected)
                wrongFlash() # calls wrongFlash function (all LEDs flash red for right choice)
                scoreDeduct() # calls scoreDeduct function (subtracts stageScore from totalScore)
                break # infinite loop broken
    elif gateLoaded == '2inOR': # if gateLoaded is 2inOR
        while True: # infinite loop until broken (break statement)
            if pb_or2.value() == True: # if pb_or2 (2inOR) selected
                rightDisp() # calls rightDisp function (displays right choice selected)
                rightFlash() # calls rightFlash function (all LEDs flash green for right choice)
                break # infinite loop broken
            elif pb_an2.value() == True or pb_an3.value() == True or pb_na2.value() == True or pb_na3.value() == True or pb_no2.value() == True or pb_no3.value() == True or pb_or3.value() == True or pb_xo2.value() == True or pb_xn2.value() == True:
                    # if pb_an2/pb_an3/pb_na2/pb_na3/pb_no2/pb_no3/pb_or3/pb_xo2/pb_xn2 selected
                wrongDisp() # calls wrongDisp function (displays wrong choice selected)
                wrongFlash() # calls wrongFlash function (all LEDs flash red for right choice)
                scoreDeduct() # calls scoreDeduct function (subtracts stageScore from totalScore)
                break # infinite loop broken
    elif gateLoaded == '3inOR': # if gateLoaded is 3inOR
        while True: # infinite loop until broken (break statement)
            if pb_or3.value() == True: # if pb_or3 (3inOR) selected
                rightDisp() # calls rightDisp function (displays right choice selected)
                rightFlash() # calls rightFlash function (all LEDs flash green for right choice)
                break # infinite loop broken
            elif pb_an2.value() == True or pb_an3.value() == True or pb_na2.value() == True or pb_na3.value() == True or pb_no2.value() == True or pb_no3.value() == True or pb_or2.value() == True or pb_xo2.value() == True or pb_xn2.value() == True:
                    # if pb_an2/pb_an3/pb_na2/pb_na3/pb_no2/pb_no2/pb_or3/pb_xo2/pb_xn2 selected
                wrongDisp() # calls wrongDisp function (displays wrong choice selected)
                wrongFlash() # calls wrongFlash function (all LEDs flash red for right choice)
                scoreDeduct() # calls scoreDeduct function (subtracts stageScore from totalScore)
                break # infinite loop broken
    elif gateLoaded == '2inXOR': # if gateLoaded is 2inXOR
        while True: # infinite loop until broken (break statement)
            if pb_xo2.value() == True: # if pb_xo2 (2inXOR) selected
                rightDisp() # calls rightDisp function (displays right choice selected)
                rightFlash() # calls rightFlash function (all LEDs flash green for right choice)
                break # infinite loop broken
            elif pb_an2.value() == True or pb_an3.value() == True or pb_na2.value() == True or pb_na3.value() == True or pb_no2.value() == True or pb_no3.value() == True or pb_or2.value() == True or pb_or3.value() == True or pb_xn2.value() == True:
                    # if pb_an2/pb_an3/pb_na2/pb_na3/pb_no2/pb_no2/pb_or2/pb_or3/pb_xn2 selected
                wrongDisp() # calls wrongDisp function (displays wrong choice selected)
                wrongFlash() # calls wrongFlash function (all LEDs flash red for right choice)
                scoreDeduct() # calls scoreDeduct function (subtracts stageScore from totalScore)
                break # infinite loop broken
    elif gateLoaded == '2inXNOR': # if gateLoaded is 2inXNOR
        while True: # infinite loop until broken (break statement)
            if pb_xn2.value() == True: # if pb_xn2 (2inXNOR) selected
                rightDisp() # calls rightDisp function (displays right choice selected)
                rightFlash() # calls rightFlash function (all LEDs flash green for right choice)
                break # infinite loop broken
            elif pb_an2.value() == True or pb_an3.value() == True or pb_na2.value() == True or pb_na3.value() == True or pb_no2.value() == True or pb_no3.value() == True or pb_or2.value() == True or pb_or3.value() == True or pb_xo2.value() == True:
                    # if pb_an2/pb_an3/pb_na2/pb_na3/pb_no2/pb_no2/pb_or2/pb_or3/pb_xo2 selected
                wrongDisp() # calls wrongDisp function (displays wrong choice selected)
                wrongFlash() # calls wrongFlash function (all LEDs flash red for right choice)
                scoreDeduct() # calls scoreDeduct function (subtracts stageScore from totalScore)
                break # infinite loop broken
    
    inpReset() # calls inpReset function to reset variables a, b, & c to 0 (inputs always start LOW)
    scoreReset() # calls scoreReset function to reset stageScore to 14 (every stage worth at most 14pts)
    playDisp() # calls playDisp function (rightDisp/wrongDisp cleared after making Gate selection)
    levelNumber = levelNumber + 1 # variable levelNumber is increased by 1 from its' previous value; this is
                                    # a crucial action that many other functions rely on in order to keep
                                    # advancing stage progression by loading next (Gate) value contained in
                                    # playLevels list, correctly updating output condition / available
                                    # inputs, etc    


######## SECTION 7 of 7: End of Game ########

for i in range(1): # for a loop of only (1) time, game end actions execute (does not happen until all 10
                    # stages/Gates ran through because game loop and only all nested contents run during so
                    # this just starts after the 10th Gate selected which nullifies the game loop and
                    # continues on after; this loop only needs to happen 1 time to set up local variables
                    # for its' own nested contents
    pb_res.irq(trigger = Pin.IRQ_FALLING, handler = inpToggler) # designates pb.res to work as an "interrupt"
                                                                # (.irq) which is a Pin class functionality;
                                                                # when triggered (on FALLING edge) will look
                                                                # to handler inpToggler (function) for
                                                                # instruction on what to execute
    inpEnd() # calls inpEnd function (sets all LEDs to staggered RGRG pattern for game end "toggle dance")
    scorePercent() # calls scorePercent function (converts remaining totalScore into a percentage for display)
    
    while True: # infinite loop until broken (has no break statement contained so will not break until
                    # pb_res activated or power turned off)
        endDisp() # calls endDisp function (displays ending totalScore & percentage right)
        endTogg() # calls endTogg function (toggles staggered LEDs from RGRG -> GRGR)
        time.sleep(1) # holds ("sleeps") endDisp & LEDs for 1 second (1000ms)
        endTogg() # calls endTogg function (toggles staggered LEDs from GRGR -> RGRG)
        time.sleep(1) # holds ("sleeps") endDisp & LEDs for 1 second (1000ms)
        funDisp() # calls funDisp function (displays fun text :D)
        endTogg() # calls endTogg function (toggles staggered LEDs from RGRG -> GRGR)
        time.sleep(1) # holds ("sleeps") endTogg & LEDs for 1 second (1000ms)
        endTogg() # calls endTogg function (toggles staggered LEDs from GRGR -> RGRG)
        time.sleep(1) # holds ("sleeps") endTogg & LEDs for 1 second (1000ms)